@@ -60,17 +60,28 @@ boost::python::tuple WrapMMCifStringToEntity(const String& mmcif,
 String WrapEntityToMMCifStringEnt(const ost::mol::EntityHandle& ent,
                                  const String& data_name,
                                  ost::conop::CompoundLibPtr compound_lib,
-                                 bool mmcif_conform) {
+                                 bool mmcif_conform,
+                                 const std::vector<MMCifWriterEntity>& entity_info) {
   return EntityToMMCifString(ent, data_name, compound_lib,
-                             mmcif_conform);
+                             mmcif_conform, entity_info);
 }
 
 String WrapEntityToMMCifStringView(const ost::mol::EntityView& ent,
                                    const String& data_name,
                                    ost::conop::CompoundLibPtr compound_lib,
-                                   bool mmcif_conform) {
+                                   bool mmcif_conform,
+                                   const std::vector<MMCifWriterEntity>& entity_info) {
   return EntityToMMCifString(ent, data_name, compound_lib,
-                             mmcif_conform);
+                             mmcif_conform, entity_info);
+}
+
+String WrapOMFToMMCifString(const ost::io::OMF& omf,
+                            const String& data_name,
+                            ost::conop::CompoundLibPtr compound_lib,
+                            bool mmcif_conform,
+                            const std::vector<MMCifWriterEntity>& entity_info){
+  return OMFToMMCifString(omf, data_name, compound_lib,
+                          mmcif_conform, entity_info);
 }
 
 void WrapStarLoopAddData(StarWriterLoop& sl, const boost::python::list& l) {
@@ -102,6 +113,14 @@ void WrapSetStructureView(MMCifWriter& writer,
   writer.SetStructure(ent, compound_lib, mmcif_conform, entity_info);
 }
 
+void WrapSetStructureOMF(MMCifWriter& writer,
+                         const ost::io::OMF& omf_ent,
+                         ost::conop::CompoundLibPtr compound_lib,
+                         bool mmcif_conform,
+                         const std::vector<MMCifWriterEntity>& entity_info) {
+  writer.SetStructure(omf_ent, compound_lib, mmcif_conform, entity_info);
+}
+
 void export_mmcif_io()
 {
   class_<MMCifReader, boost::noncopyable>("MMCifReader", init<const String&, EntityHandle&, const IOProfile&>())
@@ -181,6 +200,9 @@ void export_mmcif_io()
     .def("SetStructure", &WrapSetStructureView, (arg("ent"), arg("compound_lib"),
                                                  arg("mmcif_conform")=true,
                                                  arg("entity_info")=std::vector<MMCifWriterEntity>()))
+    .def("SetStructure", &WrapSetStructureOMF, (arg("ent"), arg("compound_lib"),
+                                                arg("mmcif_conform")=true,
+                                                arg("entity_info")=std::vector<MMCifWriterEntity>()))
     .def("GetEntities", &MMCifWriter::GetEntities, return_value_policy<copy_const_reference>())
   ;
 
@@ -578,10 +600,18 @@ void export_mmcif_io()
   def("EntityToMMCifString",  &WrapEntityToMMCifStringEnt, (arg("ent"),
                                                             arg("data_name"),
                                                             arg("compound_lib"),
-                                                            arg("mmcif_conform")));
+                                                            arg("mmcif_conform"),
+                                                            arg("entity_info")=std::vector<MMCifWriterEntity>()));
 
   def("EntityToMMCifString",  &WrapEntityToMMCifStringView, (arg("ent"),
                                                              arg("data_name"),
                                                              arg("compound_lib"),
-                                                             arg("mmcif_conform")));
+                                                             arg("mmcif_conform"),
+                                                             arg("entity_info")=std::vector<MMCifWriterEntity>()));
+
+  def("OMFToMMCifString",  &WrapOMFToMMCifString, (arg("omf"),
+                                                   arg("data_name"),
+                                                   arg("compound_lib"),
+                                                   arg("mmcif_conform"),
+                                                   arg("entity_info")=std::vector<MMCifWriterEntity>()));
 }
