@@ -122,7 +122,7 @@ void SDFReader::Import(mol::EntityHandle& ent)
 
 void SDFReader::ClearState(const boost::filesystem::path& loc)
 {
-  if (boost::iequals(".gz", boost::filesystem::extension(loc))) {
+  if (boost::iequals(".gz", loc.extension().string())) {
     in_.push(boost::iostreams::gzip_decompressor());
   }
   in_.push(instream_);
